from abc import ABC, abstractmethod, abstractproperty

import stan
import numpy as np

import warnings
import sys
import os
import contextlib

## this is to make the STAN models work in Jupyter notebooks
import asyncio
import nest_asyncio


class MCDMProblem(ABC):
    _basic_model = ""
    _basic_model_clustering = ""
    _basic_model_sorting = ""
    
    _correlated_model = ""
    _correlated_model_clustering = ""
    _correlated_model_sorting = ""

    _is_correlated_model = False
    _is_clustering_required = False
    _is_sorting_required = False

    def __init__(self, alternatives, dm_cluster_number, alt_sort_number, num_chain, num_samples, options={}):
        self.alternatives = alternatives
        self.dm_cluster_number = dm_cluster_number
        self.alt_sort_number = alt_sort_number
        self.num_chains = num_chain
        self.num_samples = num_samples
        self.options = options

        if self.alternatives is not None or self.options.get('Sigma') is not None:
            self._is_correlated_model = True
        if self.options.get('CriteriaDependence') == False:
            self._is_correlated_model = False

        self._is_sorting_required = True if self.alt_sort_number > 0 else False
        self._is_clustering_required = True if self.dm_cluster_number > 0 else False

    @property
    def alt_no(self):
        return 0 if isinstance(self.alternatives, type(None)) else self.alternatives.shape[0]

    @abstractproperty
    def input_data(self):
        pass

    @abstractproperty
    def dm_no(self):
        pass
    
    @abstractproperty
    def criteria_no(self):
        pass
    
    @abstractmethod
    def _check_input_data(self):
        pass

    @property
    def model(self):
        model = 'self._'

        model = model + 'correlated_model' if self._is_correlated_model else model + 'basic_model'
        model = model + '_clustering' if self._is_clustering_required else model
        model = model + '_sorting' if self._is_sorting_required else model

        self.model_name = model
        # print("The used model is: ", model)
        return eval(model)

    def _get_common_data(self):
        data = {}
        data['gamma_param'] = 0.1
        data['sigma_coef'] = 0.1
        data['DmNo'] = self.dm_no
        data['CNo'] = self.criteria_no
        
        if self.dm_cluster_number > 0:
            data['DmC'] = self.dm_cluster_number

        if not isinstance(self.alternatives, type(None)):
            data['Alt'] = self.alternatives
            data['AltNo'] = self.alt_no

        if self.alt_sort_number > 0:
            data['AltC'] = self.alt_sort_number
            data['eAlt'] = np.ones(self.alt_sort_number)

        if self._is_correlated_model:
            data['mu'] = 0.01 * np.ones(self.criteria_no)
            data['Sigma'] = np.cov(self.alternatives.T)
            if not isinstance(self.options.get('Sigma'), type(None)):
                data['Sigma'] = self.options.get('Sigma')
                assert data['Sigma'].shape == (self.criteria_no, self.criteria_no)

        if self.alt_sort_number > 0 and isinstance(self.alternatives, type(None)):
            raise Exception("Alternatives should be given as input for the sorting problem!")

        return data

    def sampling(self):
        if self._check_input_data:
            nest_asyncio.apply()
            asyncio.run(asyncio.sleep(1))
            with self.suppress_stan_warnings():
                posterior = stan.build(self.model, data=self.input_data, random_seed=1)
                
                print("Sampling started...")
                self.samples = posterior.sample(num_chains=self.num_chains, num_samples=self.num_samples, num_warmup=self.num_samples / 2)
                print("Sampling finished.")
                self.process_samples() 
        else:
            raise Exception("The input data is not valid")

    @contextlib.contextmanager
    def suppress_stan_warnings(self):
        import io
        f = io.StringIO()
        with contextlib.redirect_stderr(f):
            yield

    def process_samples(self):
        if self.__class__.__name__ != "StandardWeightAnalyzer":
            self.dm_weight_samples = self.samples['W'].T
            self.dm_weight = np.mean(self.dm_weight_samples, axis=0)


        if self._is_clustering_required:
            self.dm_cluster_center_samples = self.samples['wc']
            self.dm_cluster_centers = np.mean(self.dm_cluster_center_samples, axis=2)
            self.dm_membership_samples = self.samples['theta']
            self.dm_membership = np.mean(self.dm_membership_samples, axis=2)
            
        elif self._is_sorting_required:
            self.aggregated_weight_samples = self.samples['wStar']
            self.aggregated_weight = np.mean(self.aggregated_weight_samples, axis=1)
            
            soft_z_un = np.mean(self.samples['soft_z'], axis=2)
            soft_z = np.exp(soft_z_un)
            sum_soft_z = np.sum(soft_z, axis=1).reshape((self.alt_no, 1))
            self.alternative_membership = np.divide(soft_z, sum_soft_z)
            self.alternative_sorting = np.argmax(soft_z, axis=1)

            self.alternative_values = 1 / (1 + np.exp(-self.samples['v']))

            mu_un = np.mean(self.samples['altMu'], axis=1)
            self.sorting_centers = 1 / (1 + np.exp(-mu_un))
        
        else:
            self.aggregated_weight_samples = self.samples['wStar'].T
            self.aggregated_weight = np.mean(self.aggregated_weight_samples, axis=0)



import numpy as np
from Models.AHP.StandardAHP import StandardAHP
from Models.AHP.IntervalAHP import IntervalAHP
from Models.AHP.GaussianAHP import GaussianAHP
from Models.AHP.TriangularAHP import TriangularAHP

import warnings
warnings.filterwarnings("ignore", category=FutureWarning)

import logging
logging.getLogger("httpstan").setLevel(logging.ERROR)

num_samples = 200
num_chains = 2

def get_pcm_data():
    PCM =  np.array([
        [
            [1,   3,   5,     4, 7],
            [1/3, 1,   3,     2, 5],
            [1/5, 1/3, 1,   1/2, 3],
            [1/4, 1/2,   2,   1, 3],
            [1/7, 1/5, 1/3, 1/3, 1],
        ],
        [
            [1,     4,   3,    5,  8],
            [1/4,   1,   4,    3,  6],
            [1/3, 1/4,   1,    1,  5],
            [1/5, 1/3,   1,    1,  7],
            [1/8, 1/6, 1/5,  1/7,  1],
        ],
        [
            [1,   1/2,   3,   2,  5],
            [2,     1,   5,   1,  2],
            [1/3, 1/5,   1,   2,  1/2],
            [1/2,   1, 1/2,   1,  5],
            [1/5, 1/2,   2, 1/5,  1], 
        ],
        [
            [1,      3,   5,  2,  6],
            [1/3,    1,   1,  3,  2],
            [1/5,    1,   1,  4,  5],
            [1/2,  1/3, 1/4,  1,  1/2],
            [1/6,  1/2, 1/5,  2,  1],
        ],
        [
            [1,     2, 6,   3,   3],
            [1/2,   1, 2,   5,   4],
            [1/6, 1/2, 1, 1/2,   1],
            [1/3, 1/5, 2,   1,   5],
            [1/3, 1/4, 1, 1/5,   1],
        ],
        [
            [1,     2,   5,   4, 9],
            [1/2,   1,   3,   2, 6],
            [1/5, 1/3,   1,   1, 2],
            [1/4, 1/2,   1,   1, 3],
            [1/9, 1/6, 1/2, 1/3, 1],
        ]
    ])
    return PCM


def run_all_ahp_models(PCM):
    models = [
        #("StandardAHP", StandardAHP),
        #("IntervalAHP", IntervalAHP),
        #("GaussianAHP", GaussianAHP),
        ("TriangularAHP", TriangularAHP)
    ]
    for model_name, model_class in models:
        print(f"\n--- {model_name} ---")
        for crit_dep in [False]:  # , True
            for dm_cluster in [-1, 2]:
                opt = {'CriteriaDependence': crit_dep}
                print(f"CriteriaDependence={crit_dep}, dm_cluster_number={dm_cluster}")
                if model_name == "IntervalAHP":
                    ahp = model_class(
                        PCM_l=PCM,
                        PCM_h=PCM,
                        opt=opt,
                        dm_cluster_number=dm_cluster,
                        num_samples=num_samples,
                        num_chain=num_chains
                    )
                elif model_name == "GaussianAHP":
                    ahp = model_class(
                        PCM_md=PCM,
                        PCM_sigma=0.01 * np.ones_like(PCM),
                        opt=opt,
                        dm_cluster_number=dm_cluster,
                        num_samples=num_samples,
                        num_chain=num_chains
                    )
                elif model_name == "TriangularAHP":
                    ahp = model_class(
                        PCM_md=PCM,
                        opt=opt,
                        dm_cluster_number=dm_cluster,
                        num_samples=num_samples,
                        num_chain=num_chains
                    )
                elif model_name == "StandardAHP":
                    ahp = model_class(
                        PCM=PCM,
                        opt=opt,
                        dm_cluster_number=dm_cluster,
                        num_samples=num_samples,
                        num_chain=num_chains
                    )

                ahp.sampling()
        print(f"{model_name} run complete.")
        print(f"{model_name} run complete.")

if __name__ == "__main__":
    PCM = get_pcm_data()
    run_all_ahp_models(PCM)
    print("\nAll AHP model runs completed.")


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setup(
    # name="BayesMCDM",
    # version="0.1.1.57",
    # description="Bayesian Multi-Criteria Decision Making Toolkit",
    # author="Majid Mohammadi",
    # long_description=long_description,
    # long_description_content_type="text/markdown",
    # packages=find_packages(where="src"),
    # package_dir={"": "src"},
    # python_requires=">=3.9",
    # install_requires=[
    #     "numpy",
    #     "pystan",
    # ],
)   


from .StandardAHP import StandardAHP
from ..MCDMProblem import MCDMProblem
import numpy as np

class GaussianAHP(StandardAHP, MCDMProblem):
    
    _basic_model = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] matrix[CNo,CNo] PCM_md;
            array[DmNo] matrix[CNo,CNo] PCM_sigma;
            vector<lower=0,upper=1>[CNo] e;
            real<lower=0.0001> gamma_param; // prior for the Gamma distribution
        } 

        parameters {             
            simplex[CNo] wStar;
            real<lower=0> kappaStar;

            array[DmNo] simplex[CNo] W;
            array[DmNo] real<lower=0> kappa;

            array[DmNo] matrix<lower=0>[CNo,CNo] PCM;
        } 

        transformed parameters {
            array[DmNo] matrix[CNo,CNo] PCM_normalized;

            for(i in 1:DmNo) 
                for(j in 1:CNo)
                    PCM_normalized[i][,j] = col(PCM[i],j) ./  sum(col(PCM[i],j));
        }

        model {
            kappaStar ~ gamma(gamma_param,gamma_param);
            wStar ~ dirichlet(0.01*e);
            
            for (i in 1:DmNo){
                kappa[i] ~ gamma(gamma_param,gamma_param);
                W[i] ~ dirichlet(kappaStar*wStar);

                for(j in 1:CNo){
                    for (k in 1:CNo) {
                        PCM_md[i][j,k] ~ normal(PCM[i][j,k], PCM_sigma[i][j,k]);
                    }
                }
            }
        } 
    """

    _basic_model_clustering = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] matrix[CNo,CNo] PCM_md;
            array[DmNo] matrix[CNo,CNo] PCM_sigma;
            vector<lower=0,upper=1>[CNo] e;

            int<lower=2> DmC;
            real<lower=0.0001> gamma_param; // prior for the Gamma distribution
        } 

        parameters { 
            array[DmNo] simplex[CNo] W;
            array[DmNo] real<lower=0> kappa;

            array[DmC] simplex[CNo] wc;
            array[DmC] real<lower=0> ksi; 
            array[DmNo] simplex[DmC] theta;

            array[DmNo] matrix<lower=0>[CNo,CNo] PCM;
        } 

        transformed parameters {
            array[DmNo] matrix[CNo,CNo] PCM_normalized;

            for(i in 1:DmNo) 
                for(j in 1:CNo)
                    PCM_normalized[i][,j] = col(PCM[i],j) ./  sum(col(PCM[i],j));
        }

        model {      
            for(i in 1:DmC){
                wc[i] ~ dirichlet(0.01*e);
                ksi[i] ~ gamma(gamma_param,gamma_param);
            }

            for (i in 1:DmNo){
                array[DmC] real contribution;
                for(j in 1:DmC)
                     contribution[j] = log(theta[i,j]) + dirichlet_lpdf( W[i] | ksi[j]*wc[j]);
                target += log_sum_exp(contribution);

                for(j in 1:CNo)
                    for (k in 1:CNo) 
                        PCM_md[i][j,k] ~ normal(PCM[i][j,k], PCM_sigma[i][j,k]);

                kappa[i] ~ gamma(gamma_param,gamma_param);
                for(j in 1:CNo)
                    PCM_normalized[i][,j] ~ dirichlet(kappa[i]*W[i]);
                
            }
    } 
    """
    
    _basic_model_sorting = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] vector[CNo] AB_md; 
            array[DmNo] vector[CNo] AW_md; 
            array[DmNo] vector[CNo] AB_sigma; 
            array[DmNo] vector[CNo] AW_sigma; 
            vector<lower=0,upper=1>[CNo] e;

            int<lower=2> AltNo;
            int<lower=2> AltC;
            matrix[AltNo, CNo] Alt;
        } 

        parameters { 
            array[DmNo] simplex[CNo] W;
            array[DmNo] real<lower=0> kappa;

            simplex[CNo] wStar;
            real<lower=0> kappaStar;

            vector[AltC] altMu;

            array[DmNo] vector<lower=0>[CNo] AW;
            array[DmNo] vector<lower=0>[CNo] AB;
        } 

        transformed parameters {
            array[DmNo] simplex[CNo] AB_normalized;
            array[DmNo] simplex[CNo] AW_normalized;

            for(i in 1:DmNo){
                AB_normalized[i] = (e ./ AB[i]) ./ sum(e ./ AB[i]); 
                AW_normalized[i] = AW[i] ./ sum(AW[i]);
            }

            vector[AltNo] v = Alt * wStar;
            v ./= (1-v);
            v = log(v);
            array[AltNo, AltC] real<upper=0> soft_z; // log unnormalized clusters
            
            for (n in 1:AltNo)
                for (k in 1:AltC)
                    soft_z[n, k] = -log(AltC) - 0.5 * pow(altMu[k] - v[n],2);        
        }

        model {
            kappaStar ~ gamma(.01,.01);
            wStar ~ dirichlet(0.01*e);

            W ~ dirichlet(kappaStar*wStar);
            kappa ~ gamma(.01,.01);
            
            for (i in 1:DmNo){
                AB_md[i] ~ normal(AB[i], AB_sigma[i]);
                AW_md[i] ~ normal(AW[i], AW_sigma[i]);

                AW_normalized[i] ~ dirichlet(kappa[i]*W[i]);
                AB_normalized[i] ~ dirichlet(kappa[i]*W[i]);    
            }

            for (n in 1:AltNo)
                target += log_sum_exp(soft_z[n]);

        } 
    """
    
    _correlated_model = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] matrix[CNo,CNo] PCM_md;
            array[DmNo] matrix[CNo,CNo] PCM_sigma;
            vector<lower=0,upper=1>[CNo] e;

            vector[CNo] mu; // mean of unnormalized weight
            cov_matrix[CNo] Sigma;

            real<lower=0.0001> gamma_param; // prior for the Gamma distribution
            real<lower=0.001> sigma_coef; // coefficient for the covariance matrix
        } 

        parameters { 
            array[DmNo] vector[CNo] W_eta; 
            vector[CNo] wStar_eta;
            array[DmNo] real<lower=0> kappa;

            array[DmNo] matrix<lower=0>[CNo,CNo] PCM;
        } 

        transformed parameters {
            array[DmNo] matrix[CNo,CNo] PCM_normalized;

            for(i in 1:DmNo) 
                for(j in 1:CNo)
                    PCM_normalized[i][,j] = col(PCM[i],j) ./  sum(col(PCM[i],j));

            array[DmNo] simplex[CNo] W;
            simplex[CNo] wStar;

            wStar = softmax(wStar_eta);
            
            for(i in 1:DmNo)
                W[i] = softmax(W_eta[i]);
        } 

        model {
            wStar_eta ~ multi_normal(mu, Sigma);
            kappaStar ~ gamma(gamma_param,gamma_param);
            wStar ~ dirichlet(0.01*e);

            for (i in 1:DmNo){
                W_eta[i] ~ multi_normal(wStar_eta, Sigma);
                kappa[i] ~ gamma(gamma_param,gamma_param);

                for(j in 1:CNo)
                    PCM_normalized[i][,j] ~ dirichlet(kappa[i]*W[i]); 

                for(j in 1:CNo){
                    for (k in 1:CNo) {
                        PCM_md[i][j,k] ~ normal(PCM[i][j,k], PCM_sigma[i][j,k]);
                    }
                }
            }
            
        }
    """

    _correlated_model_clustering = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] matrix[CNo,CNo] PCM_md;
            array[DmNo] matrix[CNo,CNo] PCM_sigma;
            vector<lower=0,upper=1>[CNo] e;

            int<lower=2> DmC;

            vector[CNo] mu; // mean of unnormalized weight
            cov_matrix[CNo] Sigma;

            real<lower=0.0001> gamma_param; // prior for the Gamma distribution
            real<lower=0.001> sigma_coef; // coefficient for the covariance matrix
        } 

        parameters { 
            array[DmNo] vector[CNo] W_eta;
            array[DmNo] real<lower=0> kappa;

            array[DmNo] simplex[DmC] theta;
            array[DmC] vector[CNo] wc_eta;
            array[DmNo] matrix<lower=0>[CNo,CNo] PCM;
        }

        transformed parameters {
            array[DmNo] matrix[CNo,CNo] PCM_normalized;

            for(i in 1:DmNo) 
                for(j in 1:CNo)
                    PCM_normalized[i][,j] = col(PCM[i],j) ./  sum(col(PCM[i],j));
       
            array[DmNo] simplex[CNo] W;
            array[DmC] simplex[CNo] wc;

            for(i in 1:DmNo)
                W[i] = softmax(W_eta[i]);
            
            for(i in 1:DmC)
                wc[i] = softmax(wc_eta[i]);  
        } 

        model {
            for (i in 1:DmC)
                wc_eta[i] ~ multi_normal(mu, Sigma);
 
            for (i in 1:DmNo) {     
                array[DmC] real contribution; 
                for(j in 1:DmC)
                    contribution[j] = log(theta[i,j]) + multi_normal_lpdf( W_eta[i] | wc_eta[j], Sigma*sigma_coef);
                target += log_sum_exp(contribution);   

                for(j in 1:CNo){
                    for (k in 1:CNo) {
                        PCM_md[i][j,k] ~ normal(PCM[i][j,k], PCM_sigma[i][j,k]);
                    }
                }
                kappa[i] ~ gamma(gamma_param,gamma_param);
                for(j in 1:CNo){
                    PCM_normalized[i][,j] ~ dirichlet(kappa[i]*W[i]);
                }   
    
            }
        }
    """

    _correlated_model_sorting = """
        data { 
            int<lower=2> CNo;
            int<lower=1> DmNo;  
            array[DmNo] vector[CNo] AB_md; 
            array[DmNo] vector[CNo] AW_md; 
            array[DmNo] vector[CNo] AB_sigma; 
            array[DmNo] vector[CNo] AW_sigma; 
            vector<lower=0,upper=1>[CNo] e;

            int<lower=2> AltNo;
            matrix[AltNo, CNo] Alt;
            int<lower=2> AltC;
            vector<lower=0,upper=1>[AltC] eAlt;

            vector[CNo] mu; // mean of unnormalized weight
            cov_matrix[CNo] Sigma;
        } 

        parameters { 
            array[DmNo] vector[CNo] W_eta; 
            vector[CNo] wStar_eta;
            real<lower=0> kappaStar;  
            array[DmNo] real<lower=0> kappa;

            vector[AltC] altMu;

            array[DmNo] vector<lower=0>[CNo] AB;
            array[DmNo] vector<lower=0>[CNo] AW;
        } 

        transformed parameters {
            array[DmNo] simplex[CNo] AB_normalized;
            array[DmNo] simplex[CNo] AW_normalized;

            for(i in 1:DmNo){
                AB_normalized[i] = (e ./ AB[i]) ./ sum(e ./ AB[i]); 
                AW_normalized[i] = AW[i] ./ sum(AW[i]);
            }

            array[DmNo] simplex[CNo] W;
            simplex[CNo] wStar;

            wStar = softmax(wStar_eta);
            
            for(i in 1:DmNo) {
                W[i] = softmax(W_eta[i]);
            }

            vector[AltNo] v = Alt * wStar;
            v ./= (1-v);
            v = log(v);
            array[AltNo, AltC] real<upper=0> soft_z; // log unnormalized clusters
            for (n in 1:AltNo)
                for (k in 1:AltC)
                    soft_z[n, k] = -log(AltC) - 0.5 * pow(altMu[k] - v[n],2);
                
        } 

        model {
            wStar_eta ~ multi_normal(mu, Sigma);
            kappa ~ gamma(.01,.01);
                            
            for (i in 1:DmNo){
                W_eta[i] ~ multi_normal(wStar_eta, Sigma);

                AB_md[i] ~ normal(AB[i], AB_sigma[i]);
                AW_md[i] ~ normal(AW[i], AW_sigma[i]);

                AW_normalized[i] ~ dirichlet(kappa[i]*W[i]);
                AB_normalized[i] ~ dirichlet(kappa[i]*W[i]);    
            }

            for (n in 1:AltNo)
               target += log_sum_exp(soft_z[n]);
        }
    """

    def __init__(self, PCM_md, PCM_sigma, alternatives = None, dm_cluster_number=-1, alt_sort_number=-1, num_chain=3, num_samples=1000, opt={}):
        self.PCM_MD = np.array(PCM_md)
        self.PCM_Sigma = np.array(PCM_sigma)

        MCDMProblem.__init__(self, alternatives, dm_cluster_number, alt_sort_number, num_chain, num_samples,opt)

    @property
    def input_data(self):
        data = self._get_common_data()
        data['PCM_md'] = self.PCM_MD
        data['PCM_sigma'] = self.PCM_Sigma
        data['e'] =  np.ones(self.criteria_no)
        
        return data

    @property
    def dm_no(self):
        return self.PCM_MD.shape[0]
    
    @property
    def criteria_no(self):
        return self.PCM_MD.shape[1]

    def _check_input_data(self):
        assert self.PCM_MD.shape == self.PCM_Sigma.shape, "PCM_MD and PCM_Sigma must be of the same size!"

        assert self.PCM_MD.shape[0] >=1, "No input"
        assert self.PCM_MD.shape[1] >=2, "The number of criteria must be more than 2!"

        return True



import numpy as np
from Models.SWING.StandardSWING import StandardSWING
from Models.SWING.GaussianSWING import GaussianSWING
from Models.SWING.TriangularSWING import TriangularSWING
from Models.SWING.IntervalSWING import IntervalSWING

import warnings
warnings.filterwarnings("ignore", category=FutureWarning)

import logging
logging.getLogger("httpstan").setLevel(logging.ERROR)


num_samples = 200
num_chains = 2

def get_example_data():
    a_b = np.array([
        [3, 4, 6, 1, 5, 2, 9, 7],
        [1, 2, 8, 4, 5, 3, 9, 6],
        [2, 2, 3, 1, 5, 5, 9, 8],
        [2, 1, 8, 2, 9, 3, 8, 8],
        [2, 4, 9, 1, 4, 3, 5, 5],
        [1, 2, 9, 1, 3, 5, 5, 4]
    ])
    a_w = np.array([
        [7, 6, 4, 9, 5, 8, 1, 3],
        [9, 8, 2, 5, 4, 5, 1, 3],
        [8, 8, 5, 9, 5, 5, 1, 2],
        [8, 9, 2, 8, 1, 8, 2, 2],
        [8, 6, 1, 9, 6, 7, 4, 4],
        [9, 8, 1, 9, 7, 5, 5, 6]
    ])
    dmNo, cNo = a_w.shape
    altNo = 50
    x = np.random.rand(altNo // 2, cNo)
    altMat = np.concatenate([x, x * 1])
    return a_b, a_w, dmNo, cNo, altMat

def run_standard_swing(a_b, a_w, cNo, altMat):
    print("\n--- StandardSWING ---")
    for crit_dep in [False]:
        for dm_cluster in [-1,2]:
            opt = {'CriteriaDependence': crit_dep, 'Sigma': np.eye(cNo)}
            print(f"CriteriaDependence={crit_dep}")
            swing = StandardSWING(Prf=a_b*100, opt=opt, alternatives=altMat, dm_cluster_number=dm_cluster, 
                              num_samples=num_samples, num_chain=num_chains)
            swing.sampling()
    print("StandardSWING run complete.")

def run_gaussian_swing(prf, cNo, altMat):
    print("\n--- GaussianSWING ---")
    for crit_dep in [False]:
        for dm_cluster in [-1, 2]:
            opt = {'CriteriaDependence': crit_dep, 'Sigma': np.eye(cNo)}
            print(f"CriteriaDependence={crit_dep}")
            swing = GaussianSWING(
                Prf_md=prf*100, Prf_sigma=0.01*np.ones(a_b.shape),
                opt=opt, alternatives=altMat, dm_cluster_number=dm_cluster, 
                num_samples=num_samples, num_chain=num_chains
            )
            swing.sampling()
        print("GaussianSWING run complete.")

def run_triangular_swing(a_b, a_w, cNo, altMat):
    print("\n--- TriangularSWING ---")
    for crit_dep in [False]:
        for dm_cluster in [-1, 2]:
            opt = {'CriteriaIndependence': crit_dep}
            print(f"CriteriaIndependence={crit_dep}")
            swing = TriangularSWING(Prf_md=a_b, opt=opt,  dm_cluster_number=dm_cluster,
                                num_samples=num_samples, num_chain=num_chains)
            swing.sampling()
    print("TriangularSWING run complete.")

def run_interval_swing(a_b, a_w, cNo, altMat):
    print("\n--- IntervalSWING ---")
    for crit_dep in [False]:
        for dm_cluster in [-1, 2]:
            opt = {'CriteriaDependence': crit_dep}
            print(f"CriteriaDependence={crit_dep}")
            swing = IntervalSWING(
                Prf_l=a_b, Prf_h=a_b, opt=opt, 
                dm_cluster_number=dm_cluster,
                num_samples=num_samples, num_chain=num_chains
            )
            swing.sampling()
        print("IntervalSWING run complete.")

if __name__ == "__main__":
    a_b, a_w, dmNo, cNo, altMat = get_example_data()
    #run_standard_swing(a_b, a_w, cNo, altMat)
    #run_gaussian_swing(a_w, cNo, altMat) # dependece model for clustering does not work yet
    #run_triangular_swing(a_b, a_w, cNo, altMat)
    run_interval_swing(a_b, a_w, cNo, altMat)
    print("\nAll SWING model runs completed.")